/*
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2003 Ant-Contrib project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Ant-Contrib project (http://sourceforge.net/projects/ant-contrib)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The name Ant-Contrib must not be used to endorse or promote products
 *    derived from this software without prior written permission. For
 *    written permission, please contact
 *    ant-contrib-developers@lists.sourceforge.net.
 *
 * 5. Products derived from this software may not be called "Ant-Contrib"
 *    nor may "Ant-Contrib" appear in their names without prior written
 *    permission of the Ant-Contrib project.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE ANT-CONTRIB PROJECT OR ITS
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 */
 package net.sf.antcontrib.inifile;

import java.util.*;
import java.io.IOException;
import java.io.Writer;


/****************************************************************************
 * Place class description here.
 *
 * @author		inger
 * @author		<additional author>
 *
 * @since
 *
 ****************************************************************************/


public class IniSection
        implements IniPart
{
    private String name;
    private List properties;
    private Map propertyMap;

    public IniSection()
    {
        super();
        this.propertyMap = new HashMap();
        this.properties = new ArrayList();
    }


    public IniSection(String name)
    {
        this();
        this.name = name;
    }


    public List getProperties()
    {
        return properties;
    }


    public String getName()
    {
        return name;
    }


    public void setName(String name)
    {
        this.name = name;
    }


    public void write(Writer writer)
        throws IOException
    {
        writer.write("[" + name + "]");
        writer.write(System.getProperty("line.separator"));
        Iterator it = properties.iterator();
        IniProperty prop = null;
        while (it.hasNext())
        {
            prop = (IniProperty)it.next();
            prop.write(writer);
            writer.write(System.getProperty("line.separator"));
        }
    }

    public IniProperty getProperty(String name)
    {
        return (IniProperty)propertyMap.get(name);
    }

    public void setProperty(IniProperty property)
    {
        IniProperty prop = (IniProperty)propertyMap.get(property.getName());
        if (prop != null)
        {
            int idx = properties.indexOf(prop);
            properties.set(idx, property);
        }
        else
        {
            properties.add(property);
        }

        propertyMap.put(property.getName(), property);
    }

    public void removeProperty(String name)
    {
        IniProperty prop = (IniProperty)propertyMap.get(name);
        if (prop != null)
        {
            int idx = properties.indexOf(prop);
            properties.remove(idx);
            propertyMap.remove(name);
        }
    }
}
